__author__ = 'mhockenberger'

#
# Copyright 2020 Cuemacro
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the
# License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and limitations under the License.
#

"""
Shows how to calculate VWAP with finmarketpy
"""

import pandas as pd

from chartpy import Chart, Style
from finmarketpy.economics import TechIndicator, TechParams

from findatapy.util.loggermanager import LoggerManager


logger = LoggerManager().getLogger(__name__)

chart = Chart(engine='bokeh')

tech_ind = TechIndicator()

# Load data from local file
df = pd.read_csv("/Volumes/Data/s&p500.csv", index_col=0, parse_dates=['Date'],
                 date_parser=lambda x: pd.datetime.strptime(x, '%Y-%m-%d'))

# Calculate Volume Weighted Average Price (VWAP)
tech_params = TechParams()
tech_ind.create_tech_ind(df, 'VWAP', tech_params)

df = tech_ind.get_techind()

print(df)

style = Style()
style.title = 'S&P500 VWAP'
style.scale_factor = 2

df = tech_ind.get_techind()

chart.plot(df, style=style)



from setuptools import setup, find_packages

long_description = """finmarketpy is a Python based library that enables you to analyze market data and also to backtest 
trading strategies using a simple to use API, which has prebuilt templates for you to define backtest."""

setup(name='finmarketpy',
      version='0.11.7',
      description='finmarketpy is a Python based library for backtesting trading strategies',
      author='Saeed Amen',
      author_email='saeed@cuemacro.com',
      license='Apache 2.0',
      long_description=long_description,
      keywords=['trading', 'markets', 'currencies', 'pandas', 'data', 'Bloomberg', 'tick', 'stocks', 'equities'],
      url='https://github.com/cuemacro/finmarketpy',
      packages=find_packages(),
      include_package_data=True,
      install_requires=['pandas',
                        'twython',
                        'pytz',
                        'requests',
                        'numpy',
                        'multiprocess',
                        'seasonal',
                        'sklearn',
                        'matplotlib'],
      zip_safe=False)



from financepy.market.curves.FinDiscountCurveFlat import FinDiscountCurveFlat
from financepy.market.volatility.FinFXVolSurface import FinFXVolSurface
from financepy.market.volatility.FinFXVolSurface import FinFXATMMethod
from financepy.market.volatility.FinFXVolSurface import FinFXDeltaMethod

from financepy.finutils.FinDate import FinDate

import datetime
from datetime import timedelta

class FXVolSurface(object):

    def __init__(self, market_df=None, tenors=['ON', '1W', '1M', '2M', '3M', '6M', '1Y', '2Y']):
        self._market_df = market_df
        self._tenors = tenors

        self._value_date = None
        self._fin_fx_vol_surface = None

    def build_vol_surface(self, value_date, asset=None, depo_tenor='1M', field='close', atm_method=FinFXATMMethod.FWD_DELTA_NEUTRAL,
        delta_method=FinFXDeltaMethod.SPOT_DELTA):

        value_date = self._parse_date(value_date)

        self._value_date = value_date

        value_fin_date = self._findate(self._parse_date(value_date))

        tenors = self._tenors

        # Change ON (overnight) to 1D (convention for financepy)
        # tenors_financepy = list(map(lambda b: b.replace("ON", "1D"), self._tenors.copy()))
        tenors_financepy = self._tenors.copy()
        market_df = self._market_df

        field = '.' + field

        for_name_base = asset[0:3]
        dom_name_terms = asset[3:6]

        date_index = market_df.index == value_date

        # CAREFUL: need to divide by 100 for depo rate, ie. 0.0346 = 3.46%
        forCCRate = market_df[for_name_base + depo_tenor + field][date_index].values[0] / 100.0 # 0.03460  # EUR
        domCCRate = market_df[dom_name_terms + depo_tenor + field][date_index].values[0] / 100.0 # 0.02940  # USD

        dom_discount_curve = FinDiscountCurveFlat(value_fin_date, domCCRate)
        for_discount_curve = FinDiscountCurveFlat(value_fin_date, forCCRate)

        currency_pair = for_name_base + dom_name_terms
        spot_fx_rate = market_df[currency_pair + field][date_index].values[0]

        # For vols we do NOT need to divide by 100 (financepy does that internally)
        atm_vols = market_df[[currency_pair + "V" + t + field for t in tenors]][date_index].values[0]
        market_strangle25DeltaVols = market_df[[currency_pair + "25B" + t + field for t in tenors]][date_index].values[0] #[0.65, 0.75, 0.85, 0.90, 0.95, 0.85]
        risk_reversal25DeltaVols = market_df[[currency_pair + "25R" + t + field for t in tenors]][date_index].values[0] #[-0.20, -0.25, -0.30, -0.50, -0.60, -0.562]

        notional_currency = for_name_base

        # Construct financepy vol surface (uses polynomial interpolation for determining vol between strikes)
        self._fin_fx_vol_surface = FinFXVolSurface(value_fin_date,
                                   spot_fx_rate,
                                   currency_pair,
                                   notional_currency,
                                   dom_discount_curve,
                                   for_discount_curve,
                                   tenors_financepy,
                                   atm_vols,
                                   market_strangle25DeltaVols,
                                   risk_reversal25DeltaVols,
                                   atm_method,
                                   delta_method)

        #fxMarket.checkCalibration()

        # fxMarket.plotVolCurves()

        # import matplotlib.pyplot as plt
        # plt.show()

    def calculate_vol_for_strike_expiry(self, K, date):
        tenor_index = None
        return self._fin_fx_vol_surface.volFunction(self, K, tenor_index)
        #
        # TODO

    def _get_tenor_index(self, tenor):
        return self._tenors.find(tenor)

    def get_atm_strike(self, tenor=None):
        return self._fin_fx_vol_surface._K_ATM[self._get_tenor_index(tenor)]

    def get_25d_call_strike(self, tenor=None):
        return self._fin_fx_vol_surface._K_25_D_C[self._get_tenor_index(tenor)]

    def get_25d_put_strike(self, tenor=None):
        return self._fin_fx_vol_surface._K_25_D_P[self._get_tenor_index(tenor)]

    def get_10d_call_strike(self, tenor=None):
        pass

    def get_10d_put_strike(self, tenor=None):
        pass

    def get_25d_call_ms_strike(self, tenor=None):
        return self._fin_fx_vol_surface._K_25_D_C_MS[self._get_tenor_index(tenor)]

    def get_25d_put_ms_strike(self, tenor=None):
        return self._fin_fx_vol_surface._K_25_D_P_MS[self._get_tenor_index(tenor)]

    def get_10d_call_ms_strike(self, expiry_date=None, tenor=None):
        pass

    def get_10d_put_ms_strike(self, expiry_date=None, tenor=None):
        pass


    def get_atm_vol(self, expiry_date=None, tenor=None):
        pass

    def get_25d_call_vol(self, expiry_date=None, tenor=None):
        pass

    def get_25d_put_vol(self, expiry_date=None, tenor=None):
        pass

    def get_10d_call_vol(self, expiry_date=None, tenor=None):
        pass

    def get_10d_put_vol(self, expiry_date=None, tenor=None):
        pass

    def plot_vol_curves(self):
        if self._fin_fx_vol_surface is not None:
    # fxMarket.checkCalibration()
            self._fin_fx_vol_surface.plotVolCurves()

            #import matplotlib.pyplot as plt
            #plt.show()

    def _parse_date(self, date):
        if isinstance(date, str):

            date1 = datetime.datetime.utcnow()

            if date is 'midnight':
                date1 = datetime.datetime(date1.year, date1.month, date1.day, 0, 0, 0)
            elif date is 'decade':
                date1 = date1 - timedelta(days=365 * 10)
            elif date is 'year':
                date1 = date1 - timedelta(days=365)
            elif date is 'month':
                date1 = date1 - timedelta(days=30)
            elif date is 'week':
                date1 = date1 - timedelta(days=7)
            elif date is 'day':
                date1 = date1 - timedelta(days=1)
            elif date is 'hour':
                date1 = date1 - timedelta(hours=1)
            else:
                # format expected 'Jun 1 2005 01:33', '%b %d %Y %H:%M'
                try:
                    date1 = datetime.datetime.strptime(date, '%b %d %Y %H:%M')
                except:
                    # ogger.warning("Attempted to parse date")
                    i = 0

                # format expected '1 Jun 2005 01:33', '%d %b %Y %H:%M'
                try:
                    date1 = datetime.datetime.strptime(date, '%d %b %Y %H:%M')
                except:
                    # logger.warning("Attempted to parse date")
                    i = 0

                try:
                    date1 = datetime.datetime.strptime(date, '%b %d %Y')
                except:
                    # logger.warning("Attempted to parse date")
                    i = 0

                try:
                    date1 = datetime.datetime.strptime(date, '%d %b %Y')
                except:
                    # logger.warning("Attempted to parse date")
                    i = 0
        else:
            date1 = pd.Timestamp(date)

        return pd.Timestamp(date1)

    def _findate(self, date):
        return FinDate(date.day, date.month, date.year)

if __name__ == '__main__':
    from findatapy.market import Market, MarketDataRequest, MarketDataGenerator

    ticker = 'EURUSD'
    start_date = '06 Oct 2020'
    md_request = MarketDataRequest(start_date=start_date, finish_date=start_date, data_source='bloomberg', cut='LDN', category='fx-vol-market',
                                   tickers=ticker)

    import os
    import pandas as pd

    if os.path.exists(ticker + '.parquet'):
        market_df = pd.read_parquet(ticker + '.parquet')
    else:
        market_df = Market(market_data_generator=MarketDataGenerator()).fetch_market(md_request)

        market_df.to_parquet(ticker + '.parquet')

    fx_vol_surface = FXVolSurface(market_df=market_df)

    fx_vol_surface.build_vol_surface(start_date, asset='EURUSD')


import pytest
import pandas as pd
import numpy as np
from pandas.testing import assert_frame_equal
from finmarketpy.economics.techindicator import TechParams, TechIndicator

tech_params = TechParams(fillna=True, atr_period=14, sma_period=3,
                         green_n=4, green_count=9, red_n=2, red_count=13)
tech_ind = TechIndicator()
dates = pd.date_range(start='1/1/2018', end='1/08/2018')


def get_cols_name(n):
    return ['Asset%d.close' % x for x in range(1, n + 1)]


def test_sma():
    indicator_name = 'SMA'
    # Test Case 1: constant prices
    cols = get_cols_name(1)
    data_df = pd.DataFrame(index=dates, columns=cols, data=1)
    tech_ind.create_tech_ind(data_df, indicator_name, tech_params)
    expected_signal_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name, 'Signal'])
                                                            for col in cols], data=-1)
    expected_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name])
                                                     for col in cols], data=1)
    expected_signal_df.iloc[:tech_params.sma_period] = np.nan
    expected_df.iloc[:tech_params.sma_period - 1] = np.nan
    df = tech_ind.get_techind()
    signal_df = tech_ind.get_signal()

    assert_frame_equal(df, expected_df)
    assert_frame_equal(signal_df, expected_signal_df)

    # Test Case 2: Normal case with one single security
    data_df = pd.DataFrame(index=dates, columns=cols, data=list(range(1, 9)))
    tech_ind.create_tech_ind(data_df, indicator_name, tech_params)
    expected_signal_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name, 'Signal'])
                                                            for col in cols], data=1)
    expected_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name])
                                                     for col in cols], data=data_df.shift().values)
    df = tech_ind.get_techind()
    signal_df = tech_ind.get_signal()

    expected_signal_df.iloc[:tech_params.sma_period] = np.nan
    expected_df.iloc[:tech_params.sma_period - 1] = np.nan

    assert_frame_equal(df, expected_df)
    assert_frame_equal(signal_df, expected_signal_df)

    # Test Case 3: Normal case with multiple securities
    cols = get_cols_name(10)
    col_prices = np.array(range(1, 9))
    data_df = pd.DataFrame(index=dates, columns=cols, data=np.tile(col_prices, (len(cols), 1)).T)
    tech_ind.create_tech_ind(data_df, indicator_name, tech_params)
    expected_signal_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name, 'Signal'])
                                                            for col in cols], data=1)
    expected_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name])
                                                     for col in cols], data=data_df.shift().values)
    df = tech_ind.get_techind()
    signal_df = tech_ind.get_signal()

    expected_signal_df.iloc[:tech_params.sma_period] = np.nan
    expected_df.iloc[:tech_params.sma_period - 1] = np.nan

    assert_frame_equal(df, expected_df)
    assert_frame_equal(signal_df, expected_signal_df)

    # Test Case 4: Decreasing price with multiple securities
    cols = get_cols_name(10)
    col_prices = np.array(range(8, 0, -1))
    data_df = pd.DataFrame(index=dates, columns=cols, data=np.tile(col_prices, (len(cols), 1)).T)
    tech_ind.create_tech_ind(data_df, indicator_name, tech_params)
    expected_signal_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name, 'Signal'])
                                                            for col in cols], data=-1)
    expected_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name])
                                                     for col in cols], data=data_df.shift().values)
    df = tech_ind.get_techind()
    signal_df = tech_ind.get_signal()

    expected_signal_df.iloc[:tech_params.sma_period] = np.nan
    expected_df.iloc[:tech_params.sma_period - 1] = np.nan

    assert_frame_equal(df, expected_df)
    assert_frame_equal(signal_df, expected_signal_df)

    # Test Case 5: With SOME missing data
    cols = get_cols_name(1)
    data_df = pd.DataFrame(index=dates, columns=cols, data=list(range(1, 9)))
    data_df.iloc[3] = np.nan

    tech_ind.create_tech_ind(data_df, indicator_name, tech_params)
    expected_signal_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name, 'Signal'])
                                                            for col in cols], data=1)
    expected_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name])
                                                     for col in cols], data=[np.nan, np.nan, 2, 2.67, 3.67,
                                                                             4.67, 6, 7])
    df = tech_ind.get_techind()
    signal_df = tech_ind.get_signal()

    expected_signal_df.iloc[:tech_params.sma_period] = np.nan

    assert_frame_equal(df.apply(lambda x: round(x, 2)), expected_df)
    assert_frame_equal(signal_df, expected_signal_df)

    # Test Case 6: With not enough data
    data_df = pd.DataFrame(index=dates, columns=cols, data=list(range(1, 9)))
    tech_params.sma_period = 20

    tech_ind.create_tech_ind(data_df, indicator_name, tech_params)
    expected_signal_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name, 'Signal'])
                                                            for col in cols], data=np.nan)
    expected_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name])
                                                     for col in cols], data=np.nan)
    df = tech_ind.get_techind()
    signal_df = tech_ind.get_signal()

    assert_frame_equal(df, expected_df)
    assert_frame_equal(signal_df, expected_signal_df)


def test_roc():
    indicator_name = 'ROC'
    # Test Case 1: constant prices
    cols = get_cols_name(1)
    tech_params.roc_period = 3
    data_df = pd.DataFrame(index=dates, columns=cols, data=1)
    tech_ind.create_tech_ind(data_df, indicator_name, tech_params)
    expected_signal_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name, 'Signal'])
                                                            for col in cols], data=-1)
    expected_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name])
                                                     for col in cols], data=0)
    expected_signal_df.iloc[:tech_params.roc_period] = np.nan
    expected_df.iloc[:tech_params.roc_period] = np.nan
    df = tech_ind.get_techind()
    signal_df = tech_ind.get_signal()

    assert_frame_equal(df, expected_df)
    assert_frame_equal(signal_df, expected_signal_df)

    # Test Case 2: Increasing prices, fixed rate
    cols = get_cols_name(1)
    tech_params.roc_period = 1
    data_df = pd.DataFrame(index=dates, columns=cols, data=[1, 2, 4, 8, 16, 32, 64, 128])
    tech_ind.create_tech_ind(data_df, indicator_name, tech_params)
    expected_signal_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name, 'Signal'])
                                                            for col in cols], data=1)
    expected_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name])
                                                     for col in cols], data=1)
    expected_signal_df.iloc[:tech_params.roc_period] = np.nan
    expected_df.iloc[:tech_params.roc_period] = np.nan
    df = tech_ind.get_techind()
    signal_df = tech_ind.get_signal()

    assert_frame_equal(df, expected_df)
    assert_frame_equal(signal_df, expected_signal_df)

    tech_params.roc_period = 2
    data_df = pd.DataFrame(index=dates, columns=cols, data=[1, 2, 4, 8, 16, 32, 64, 128])
    tech_ind.create_tech_ind(data_df, indicator_name, tech_params)
    expected_signal_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name, 'Signal'])
                                                            for col in cols], data=1)
    expected_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name])
                                                     for col in cols], data=3)
    expected_signal_df.iloc[:tech_params.roc_period] = np.nan
    expected_df.iloc[:tech_params.roc_period] = np.nan
    df = tech_ind.get_techind()
    signal_df = tech_ind.get_signal()

    assert_frame_equal(df, expected_df)
    assert_frame_equal(signal_df, expected_signal_df)

    tech_params.roc_period = 3
    data_df = pd.DataFrame(index=dates, columns=cols, data=[1, 2, 4, 8, 16, 32, 64, 128])
    tech_ind.create_tech_ind(data_df, indicator_name, tech_params)
    expected_signal_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name, 'Signal'])
                                                            for col in cols], data=1)
    expected_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name])
                                                     for col in cols], data=7)
    expected_signal_df.iloc[:tech_params.roc_period] = np.nan
    expected_df.iloc[:tech_params.roc_period] = np.nan
    df = tech_ind.get_techind()
    signal_df = tech_ind.get_signal()

    assert_frame_equal(df, expected_df)
    assert_frame_equal(signal_df, expected_signal_df)

    # Test Case 3: Missing values
    cols = get_cols_name(1)
    tech_params.roc_period = 1
    data_df = pd.DataFrame(index=dates, columns=cols, data=[1, 2, 4, 8, 16, 32, 64, 128])
    data_df.iloc[3] = np.nan
    tech_ind.create_tech_ind(data_df, indicator_name, tech_params)
    expected_signal_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name, 'Signal'])
                                                            for col in cols], data=[1, 1, 1, -1, 1, 1, 1, 1])
    expected_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name])
                                                     for col in cols], data=[1, 1, 1, 0, 3, 1, 1, 1])
    expected_signal_df.iloc[:tech_params.roc_period] = np.nan
    expected_df.iloc[:tech_params.roc_period] = np.nan
    df = tech_ind.get_techind()
    signal_df = tech_ind.get_signal()

    assert_frame_equal(df, expected_df)
    assert_frame_equal(signal_df, expected_signal_df)


def test_sma2():
    indicator_name = 'SMA2'
    tech_params.sma_period = 2
    tech_params.sma2_period = 3
    # Test Case 1: Increasing prices
    cols = get_cols_name(1)
    signals = ['SMA', 'SMA2']
    data_df = pd.DataFrame(index=dates, columns=cols, data=list(range(1, 9)))
    tech_ind.create_tech_ind(data_df, indicator_name, tech_params)
    expected_signal_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name, 'Signal'])
                                                            for col in cols], data=1)
    expected_df = pd.DataFrame(index=dates, columns=[' '.join([col, sig])
                                                     for col in cols for sig in signals],
                               data=[
                                   [np.nan, np.nan],
                                   [1.5, np.nan],
                                   [2.5, 2.0],
                                   [3.5, 3.0],
                                   [4.5, 4.0],
                                   [5.5, 5.0],
                                   [6.5, 6.0],
                                   [7.5, 7.0],
                               ])
    expected_signal_df.iloc[:tech_params.sma2_period] = np.nan
    df = tech_ind.get_techind()
    signal_df = tech_ind.get_signal()

    assert_frame_equal(df, expected_df)
    assert_frame_equal(signal_df, expected_signal_df)

    # Test Case 2: Decreasing prices
    cols = get_cols_name(1)
    signals = ['SMA', 'SMA2']
    data_df = pd.DataFrame(index=dates, columns=cols, data=list(range(8, 0, -1)))
    tech_ind.create_tech_ind(data_df, indicator_name, tech_params)
    expected_signal_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name, 'Signal'])
                                                            for col in cols], data=-1)
    expected_df = pd.DataFrame(index=dates, columns=[' '.join([col, sig])
                                                     for col in cols for sig in signals],
                               data=[
                                   [np.nan, np.nan],
                                   [7.5, np.nan],
                                   [6.5, 7.0],
                                   [5.5, 6.0],
                                   [4.5, 5.0],
                                   [3.5, 4.0],
                                   [2.5, 3.0],
                                   [1.5, 2.0],
                               ])
    expected_signal_df.iloc[:tech_params.sma2_period] = np.nan
    df = tech_ind.get_techind()
    signal_df = tech_ind.get_signal()

    assert_frame_equal(df, expected_df)
    assert_frame_equal(signal_df, expected_signal_df)

    # Test Case 3: Costant prices
    cols = get_cols_name(1)
    signals = ['SMA', 'SMA2']
    data_df = pd.DataFrame(index=dates, columns=cols, data=1)
    tech_ind.create_tech_ind(data_df, indicator_name, tech_params)
    expected_signal_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name, 'Signal'])
                                                            for col in cols], data=-1)
    expected_df = pd.DataFrame(index=dates, columns=[' '.join([col, sig])
                                                     for col in cols for sig in signals],
                               data=np.tile(np.ones(len(dates)), (2, 1)).T)
    expected_signal_df.iloc[:tech_params.sma2_period] = np.nan
    expected_df.iloc[:tech_params.sma2_period - 1] = np.nan
    expected_df.set_value('2018-01-02', 'Asset1.close SMA', 1.0)
    df = tech_ind.get_techind()
    signal_df = tech_ind.get_signal()

    assert_frame_equal(df, expected_df)
    assert_frame_equal(signal_df, expected_signal_df)


def test_polarity():
    indicator_name = 'polarity'
    # Test Case 1: constant prices
    cols = get_cols_name(1)
    data_df = pd.DataFrame(index=dates, columns=cols, data=1)
    tech_ind.create_tech_ind(data_df, indicator_name, tech_params)
    expected_signal_df = pd.DataFrame(index=dates, columns=[' '.join([col, 'Polarity', 'Signal'])
                                                            for col in cols], data=1)
    expected_df = pd.DataFrame(index=dates, columns=[' '.join([col, 'Polarity'])
                                                     for col in cols], data=1)
    df = tech_ind.get_techind()
    signal_df = tech_ind.get_signal()

    assert_frame_equal(df, expected_df)
    assert_frame_equal(signal_df, expected_signal_df, check_dtype=False)

    # Test Case 2: Increasing prices
    cols = get_cols_name(1)
    data_df = pd.DataFrame(index=dates, columns=cols, data=[1, 2, 4, 8, 16, 32, 64, 128])
    tech_ind.create_tech_ind(data_df, indicator_name, tech_params)
    expected_signal_df = pd.DataFrame(index=dates, columns=[' '.join([col, 'Polarity', 'Signal'])
                                                            for col in cols], data=1)
    expected_df = pd.DataFrame(index=dates, columns=[' '.join([col, 'Polarity'])
                                                     for col in cols], data=data_df.values)
    df = tech_ind.get_techind()
    signal_df = tech_ind.get_signal()

    assert_frame_equal(df, expected_df)
    assert_frame_equal(signal_df, expected_signal_df, check_dtype=False)

    # Test Case 3: Decreasing prices
    cols = get_cols_name(1)
    data_df = pd.DataFrame(index=dates, columns=cols, data=list(range(-1, -9, -1)))
    tech_ind.create_tech_ind(data_df, indicator_name, tech_params)
    expected_signal_df = pd.DataFrame(index=dates, columns=[' '.join([col, 'Polarity', 'Signal'])
                                                            for col in cols], data=-1)
    expected_df = pd.DataFrame(index=dates, columns=[' '.join([col, 'Polarity'])
                                                     for col in cols], data=data_df.values)
    df = tech_ind.get_techind()
    signal_df = tech_ind.get_signal()

    assert_frame_equal(df, expected_df)
    assert_frame_equal(signal_df, expected_signal_df, check_dtype=False)

    # Test Case 4: Missing values
    cols = get_cols_name(1)
    tech_params.roc_period = 1
    data_df = pd.DataFrame(index=dates, columns=cols, data=[1, 2, 4, 8, 16, 32, 64, 128])
    data_df.iloc[3] = np.nan
    tech_ind.create_tech_ind(data_df, indicator_name, tech_params)
    expected_signal_df = pd.DataFrame(index=dates, columns=[' '.join([col, 'Polarity', 'Signal'])
                                                            for col in cols], data=1)
    expected_df = pd.DataFrame(index=dates, columns=[' '.join([col, 'Polarity'])
                                                     for col in cols],
                               data=[1.0, 2.0, 4.0, 4.0, 16.0, 32.0, 64.0, 128.0])
    df = tech_ind.get_techind()
    signal_df = tech_ind.get_signal()

    assert_frame_equal(df, expected_df)
    assert_frame_equal(signal_df, expected_signal_df, check_dtype=False)


def test_attr():
    """
    Testing of attributes such as long only
    """
    indicator_name = 'SMA'
    tech_params = TechParams()
    # Test Case 1: Only longs or only shorts
    # Test Case 1.1: Only longs
    cols = get_cols_name(1)
    tech_params.sma_period = 3
    tech_params.only_allow_longs = True
    data_df = pd.DataFrame(index=dates, columns=cols, data=list(range(1, 9)))
    tech_ind.create_tech_ind(data_df, indicator_name, tech_params)
    expected_signal_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name, 'Signal'])
                                                            for col in cols], data=1)
    expected_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name])
                                                     for col in cols], data=data_df.shift().values)
    df = tech_ind.get_techind()
    signal_df = tech_ind.get_signal()

    expected_signal_df.iloc[:tech_params.sma_period] = np.nan
    expected_df.iloc[:tech_params.sma_period - 1] = np.nan

    assert_frame_equal(df, expected_df)
    assert_frame_equal(signal_df, expected_signal_df)

    data_df = pd.DataFrame(index=dates, columns=cols, data=list(range(8, 0, -1)))
    tech_ind.create_tech_ind(data_df, indicator_name, tech_params)
    expected_signal_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name, 'Signal'])
                                                            for col in cols], data=0)
    expected_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name])
                                                     for col in cols], data=data_df.shift().values)
    df = tech_ind.get_techind()
    signal_df = tech_ind.get_signal()

    expected_signal_df.iloc[:tech_params.sma_period] = np.nan
    expected_df.iloc[:tech_params.sma_period - 1] = np.nan

    assert_frame_equal(df, expected_df)
    assert_frame_equal(signal_df, expected_signal_df)

    # Test Case 1.1: Only shorts
    tech_params = TechParams()
    tech_params.sma_period = 3
    tech_params.only_allow_shorts = True
    data_df = pd.DataFrame(index=dates, columns=cols, data=list(range(1, 9)))
    tech_ind.create_tech_ind(data_df, indicator_name, tech_params)
    expected_signal_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name, 'Signal'])
                                                            for col in cols], data=0)
    expected_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name])
                                                     for col in cols], data=data_df.shift().values)
    df = tech_ind.get_techind()
    signal_df = tech_ind.get_signal()

    expected_signal_df.iloc[:tech_params.sma_period] = np.nan
    expected_df.iloc[:tech_params.sma_period - 1] = np.nan

    assert_frame_equal(df, expected_df)
    assert_frame_equal(signal_df, expected_signal_df)

    data_df = pd.DataFrame(index=dates, columns=cols, data=list(range(8, 0, -1)))
    tech_ind.create_tech_ind(data_df, indicator_name, tech_params)
    expected_signal_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name, 'Signal'])
                                                            for col in cols], data=-1)
    expected_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name])
                                                     for col in cols], data=data_df.shift().values)
    df = tech_ind.get_techind()
    signal_df = tech_ind.get_signal()

    expected_signal_df.iloc[:tech_params.sma_period] = np.nan
    expected_df.iloc[:tech_params.sma_period - 1] = np.nan

    assert_frame_equal(df, expected_df)
    assert_frame_equal(signal_df, expected_signal_df)

    # Test Case 2: Check incompatibility between only_allow_longs and only_allow_shorts

    tech_params = TechParams()
    tech_params.only_allow_shorts = True
    with pytest.raises(Exception):
        tech_params.only_allow_longs = True

    tech_params = TechParams()
    tech_params.only_allow_longs = True
    with pytest.raises(Exception):
        tech_params.only_allow_shorts = True

    # Test Case 3: test strip_signal_name
    tech_params = TechParams()
    tech_params.sma_period = 3
    tech_params.strip_signal_name = True
    data_df = pd.DataFrame(index=dates, columns=cols, data=list(range(1, 9)))
    tech_ind.create_tech_ind(data_df, indicator_name, tech_params)
    expected_signal_df = pd.DataFrame(index=dates, columns=cols, data=1)
    expected_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name])
                                                     for col in cols], data=data_df.shift().values)
    df = tech_ind.get_techind()
    signal_df = tech_ind.get_signal()

    expected_signal_df.iloc[:tech_params.sma_period] = np.nan
    expected_df.iloc[:tech_params.sma_period - 1] = np.nan

    assert_frame_equal(df, expected_df)
    assert_frame_equal(signal_df, expected_signal_df)

    # Test Case 4: test signal multiplier
    tech_params = TechParams()
    tech_params.sma_period = 3
    tech_params.signal_mult = -1
    data_df = pd.DataFrame(index=dates, columns=cols, data=list(range(1, 9)))
    tech_ind.create_tech_ind(data_df, indicator_name, tech_params)
    expected_signal_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name, 'Signal'])
                                                            for col in cols], data=-1)
    expected_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name])
                                                     for col in cols], data=data_df.shift().values)
    df = tech_ind.get_techind()
    signal_df = tech_ind.get_signal()

    expected_signal_df.iloc[:tech_params.sma_period] = np.nan
    expected_df.iloc[:tech_params.sma_period - 1] = np.nan

    assert_frame_equal(df, expected_df)
    assert_frame_equal(signal_df, expected_signal_df)

    tech_params.signal_mult = 0
    data_df = pd.DataFrame(index=dates, columns=cols, data=list(range(1, 9)))
    tech_ind.create_tech_ind(data_df, indicator_name, tech_params)
    expected_signal_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name, 'Signal'])
                                                            for col in cols], data=0)
    expected_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name])
                                                     for col in cols], data=data_df.shift().values)
    df = tech_ind.get_techind()
    signal_df = tech_ind.get_signal()

    expected_signal_df.iloc[:tech_params.sma_period] = np.nan
    expected_df.iloc[:tech_params.sma_period - 1] = np.nan

    assert_frame_equal(df, expected_df)
    assert_frame_equal(signal_df, expected_signal_df)

    tech_params.signal_mult = 3
    data_df = pd.DataFrame(index=dates, columns=cols, data=list(range(1, 9)))
    tech_ind.create_tech_ind(data_df, indicator_name, tech_params)
    expected_signal_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name, 'Signal'])
                                                            for col in cols], data=3)
    expected_df = pd.DataFrame(index=dates, columns=[' '.join([col, indicator_name])
                                                     for col in cols], data=data_df.shift().values)
    df = tech_ind.get_techind()
    signal_df = tech_ind.get_signal()

    expected_signal_df.iloc[:tech_params.sma_period] = np.nan
    expected_df.iloc[:tech_params.sma_period - 1] = np.nan

    assert_frame_equal(df, expected_df)
    assert_frame_equal(signal_df, expected_signal_df)
